use std::collections::{HashMap, VecDeque};
use chrono::{DateTime, Utc};
use log::{Level};
use systemd_journal_logger::connected_to_journal;

#[derive(Clone)]
pub struct Logging {
    mem_log: HashMap<Level, VecDeque<LogEvent>>,
    mem_log_lengths: HashMap<Level, i8>,
}

impl Logging {
    pub fn new(log_lengths: Option<HashMap<Level, i8>>) -> Self {
        let mut mem_log = HashMap::new();
        let mut mem_log_lengths = HashMap::new();

        Level::iter().for_each(|level| {
            let mem_vec = VecDeque::new();
            mem_log.insert(level, mem_vec);

            let length = match level {
                Level::Error => -1,
                Level::Warn => 40,
                Level::Info => 20,
                Level::Debug => 10,
                Level::Trace => 5,
            };
            mem_log_lengths.insert(level, length);
        });
        
        if let Some(lengths) = log_lengths {
            lengths.iter().for_each(|(level, length)| {
                mem_log_lengths.insert(*level, *length);
            });
        }

        Self {
            mem_log,
            mem_log_lengths
        }
    }
    pub fn debug(&mut self, msg: String) {
        let msg = format!("[DEBUG] {msg}");
        match connected_to_journal() {
            true => log::debug!("{msg}"),
            false => println!("{msg}"),
        }
        self.mem_log(Level::Debug, Some(msg));
    }

    pub fn info(&mut self, msg: String) {
        let msg = format!("[INFO] {msg}");
        match connected_to_journal() {
            true => log::info!("{msg}"),
            false => println!("{msg}"),
        }
        self.mem_log(Level::Info, Some(msg));
    }

    pub fn warn(&mut self, msg: String) {
        let msg = format!("[WARN] {msg}");
        match connected_to_journal() {
            true => log::warn!("{msg}"),
            false => println!("{msg}"),
        }
        self.mem_log(Level::Warn, Some(msg));
    }

    pub fn error(&mut self, msg: String) {
        let msg = format!("[ERROR] {msg}");
        match connected_to_journal() {
            true => log::error!("{msg}"),
            false => eprintln!("{msg}"),
        }
        self.mem_log(Level::Error, Some(msg));
    }

    pub fn get_mem_log(&mut self, level: Level) -> VecDeque<LogEvent> {
        self.mem_log(level, None)
    }

    fn mem_log(&mut self, level: Level, msg: Option<String>) -> VecDeque<LogEvent> {
        let max_len = self.mem_log_lengths.get(&level).expect("All Levels Should Have Lengths Initialized");
        let list = self.mem_log.get_mut(&level).expect("All Levels Should Have Empty Lists Initialized");
        
        if let Some(msg) = msg {
            let now = Utc::now();
            let log_event = LogEvent::new(now, msg);
            list.push_back(log_event);
            if *max_len != -1 {
                while list.len() > *max_len as usize {
                    list.pop_front();
                }
            }
        }

        list.clone()
    }
}

#[derive(Clone)]
pub(crate) struct LogEvent {
    pub date: DateTime<Utc>,
    pub text: String,
}

impl LogEvent {
    pub fn new(time: DateTime<Utc>, message: String) -> Self {
        Self {
            date: time,
            text: message
        }
    }
}